@@ -8,6 +8,8 @@ import argparse
 from hashlib import sha256
 from typing import List, Any, Dict
 
+from firelab.utils.data_utils import text_to_markdown
+
 
 CONFIG_ARG_PREFIX = '--config.'
 ALLOWED_LIST_SEPARATORS = [',', ' ', '|', '-']
@@ -194,7 +196,7 @@ class Config:
 
         return result
 
-    def save(self, save_path:os.PathLike, parents:bool=True):
+    def save(self, save_path: os.PathLike, parents: bool=True):
         """Saves config in the specified path"""
         if parents and not os.path.exists(os.path.dirname(save_path)):
             os.makedirs(os.path.dirname(save_path), exist_ok=True)
@@ -230,7 +232,9 @@ class Config:
         Converts the config into a string which is appropriately displayed
         in markdown (useful for tensorboard logging)
         """
-        raise NotImplementedError
+        config_yml = yaml.safe_dump(self.to_dict())
+
+        return text_to_markdown(config_yml)
 
 
 def homogenous_array_message(array: List) -> str:
