@@ -29,29 +29,23 @@ def run(cmd: str, args):
     if cmd == 'start':
         config = create_new_experiment(args)
         start_experiment(config, tb_port=args.tb_port, stay_after_training=args.stay_after_training)
-    elif cmd == 'continue':
-        #continue_experiment(init_config(args), args)
-        raise NotImplementedError
-    elif cmd == 'tb':
-        run_tensorboard_for_exp(args)
-    elif cmd == 'ls':
-        raise NotImplementedError
-    elif cmd == 'clean':
-        clean_experiments_by_prefix(construct_full_exp_dir(args.exp_dirname), args.prefix)
     else:
         raise NotImplementedError
 
 
 def create_new_experiment(args):
-    # TODO: looks like this lines should not be here
-    exp_dir = construct_full_exp_dir(args.exp_dirname)
-    os.makedirs(exp_dir, exist_ok=True)
-    config_name = os.path.basename(args.config_path)[:-4]
-    exp_name = args.exp_name if args.exp_name else config_name
-    version = infer_new_experiment_version(exp_dir, exp_name)
-    exp_name = f'{exp_name}-{version:05d}'
+    if args.experiment_dir is None:
+        # Creating a proper experiment name from config
+        config_name = os.path.basename(args.config_path)[:-4]
+        exp_name = args.exp_name if args.exp_name else config_name
+        version = infer_new_experiment_version('experiments', exp_name)
 
-    config = init_config(args.config_path, exp_dir, exp_name)
+        experiment_dir = f'experiments/{exp_name}-{version:05d}'
+    else:
+        experiment_dir = args.experiment_dir
+
+    full_experiment_dir = construct_full_path(experiment_dir)
+    config = init_config(args.config_path, full_experiment_dir)
 
     # TODO: Trainer should do this thing, no?
     # shutil.copyfile(args.config_path, config.firelab.paths.config_path)
@@ -66,15 +60,15 @@ def start_experiment(config, tb_port: int=None, stay_after_training: bool=False)
         check_that_path_exists(config.firelab.paths.checkpoints_path)
         # check_that_path_exists(config.firelab.summary_path) # TODO
 
-    if tb_port:
-        logger.info(f'Starting tensorboard on port {tb_port}')
-        run_tensorboard(config.firelab.paths.logs_path, tb_port)
-
     # TODO: are there any better ways to reach src.trainers?
     sys.path.append(os.getcwd())
     trainers = importlib.import_module('src.trainers')
     TrainerClass = getattr(trainers, config.get('trainer'))
 
+    if tb_port:
+        logger.info(f'Starting tensorboard on port {tb_port}')
+        run_tensorboard(config.firelab.experiment_dir, tb_port)
+
     if config.has('hpo'):
         if config.firelab.has('continue_from_iter'):
             raise NotImplementedError("Continuing HPO is not yet implemented")
@@ -83,6 +77,7 @@ def start_experiment(config, tb_port: int=None, stay_after_training: bool=False)
     else:
         trainer = TrainerClass(config)
         trainer.start()
+
         del trainer # To release memory (someday)
 
     # if stay_after_training:
@@ -90,11 +85,11 @@ def start_experiment(config, tb_port: int=None, stay_after_training: bool=False)
     #     signal.pause()
 
 
-def run_hpo(TrainerClass, global_config):
+def run_hpo(TrainerClass, global_config: Config):
     # TODO: Is it ok to assume that we always have more CPUs than concurrent experiments?
     configs = spawn_configs_for_hpo(global_config)
-    clean_dir(os.path.join(global_config.firelab.paths.experiments_dir,
-                           global_config.firelab.exp_name, 'summaries'), create=True)
+    clean_dir(os.path.join(
+        global_config.experiment_dir, global_config.exp_name, 'summaries'), create=True)
 
     n_parallel = global_config.hpo.get('num_parallel_experiments_per_gpu', 1) \
                * (len(global_config.firelab.gpus) \
@@ -162,28 +157,7 @@ def run_single_hpo_experiment(TrainerClass:BaseTrainer,
                 gpus_usage[gpu_idx] -= 1
 
 
-# def continue_experiment(config, args):
-#     # Finding latest checkpoint
-#     checkpoints = os.listdir(config.firelab.paths.checkpoints_path)
-
-#     if checkpoints == []:
-#         raise Exception('Can\'t continue: no checkpoints are available')
-
-#     if args.iteration is None:
-#         iters = [int(c.split('.')[0].split('-')[-1]) for c in checkpoints]
-#         iteration = max(iters)
-#     else:
-#         iteration = args.iteration
-
-#     logger.info('Continuing from iteration #{}.'.format(iteration))
-#     config.firelab.set('continue_from_iter', iteration)
-#     config.firelab.set('reset_iters_counter', args.reset_iters_counter)
-
-#     start_experiment(config, args)
-
-
-def init_config(config_path: str, exp_dirname: str, exp_name: str):
-    paths = compute_paths(exp_dirname, exp_name)
+def init_config(config_path: str, full_experiment_dir: str) -> Config:
     config = Config.load(config_path)
     config = config.overwrite(Config.read_from_cli())
 
@@ -194,16 +168,8 @@ def init_config(config_path: str, exp_dirname: str, exp_name: str):
     # Let's augment config with some helping stuff
     # TODO: assign paths automatically, because we have a lot of duplication
     config.set('firelab', {
-        'exp_name': exp_name,
-        'paths': {
-            'config_path': paths['config_path'],
-            'project_path': os.getcwd(),
-            'experiments_dir': paths['experiments_dir'],
-            'logs_path': paths['logs_path'],
-            'checkpoints_path': paths['checkpoints_path'],
-            'summary_path': paths['summary_path'],
-            'custom_data_path': paths['custom_data_path']
-        }
+        'exp_name': os.path.basename(os.path.normpath(full_experiment_dir)),
+        'experiment_dir': full_experiment_dir
     })
 
     if config.has('random_seed'):
@@ -233,37 +199,10 @@ def init_config(config_path: str, exp_dirname: str, exp_name: str):
     return config
 
 
-def compute_paths(exp_dirname, exp_name):
-    "Calculates paths for a given experiment"
-    experiments_dir = construct_full_exp_dir(exp_dirname)
-
-    return {
-        'experiments_dir': experiments_dir,
-        'config_path': os.path.join(experiments_dir, exp_name, "config.yml"),
-        'logs_path': os.path.join(experiments_dir, exp_name, "logs"),
-        'checkpoints_path': os.path.join(experiments_dir, exp_name, "checkpoints"),
-        'summary_path': os.path.join(experiments_dir, exp_name, "summary.yml"),
-        'custom_data_path': os.path.join(experiments_dir, exp_name, 'custom_data')
-    }
-
-
-def construct_full_exp_dir(exp_dirname: str) -> os.PathLike:
+def construct_full_path(dirname: str) -> os.PathLike:
     # TODO: We can't rely on os.getcwd(). How to get project dir properly?
-    return os.path.join(os.getcwd(), exp_dirname)
-
-
-def run_tensorboard_for_exp(args):
-    config_path = compute_paths(args.exp_dirname, args.exp_name)['config_path']
-    config = init_config(config_path, construct_full_exp_dir(args.exp_dirname), args.exp_name)
-    run_tensorboard(config.firelab.paths.logs_path, args.tb_port)
-    signal.pause()
-
-
-def clean_experiments_by_prefix(experiments_dir: str, prefix: str):
-    for dir in os.listdir(experiments_dir):
-        if dir.startswith(prefix):
-            dir_path = os.path.join(experiments_dir, dir)
-            logger.info(f'Removing {dir_path}')
-            shutil.rmtree(dir_path)
-
-    logger.info('Done')
+    if dirname.startswith("/"):
+        return dirname
+    else:
+        # Ok, the user is very lazy and wants us to do everything ourselves...
+        return os.path.join(os.getcwd(), dirname)
